package com.cloud.aop;

import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.core.annotation.DispersedTransaction;
import com.cloud.framework.core.annotation.RequestSubmitToken;
import com.cloud.framework.util.IpUtil;
import com.cloud.web.util.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Objects;

/**
 * @Author sj
 * Date on 2020/11/30  10:37
 * @Version 1.0
 * 重复提交表单验证
 */
@Slf4j
@Aspect
@Component
public class NoRepeatSubmitAop {
    @Autowired
    private RedisUtils redisUtil;

    /**
     * <p> 【环绕通知】 用于拦截指定方法，判断用户表单保存操作是否属于重复提交 <p>
     * <p>
     * 定义切入点表达式： execution(public * (…))
     * 表达式解释： execution：主体    public:可省略   *：标识方法的任意返回值  任意包+类+方法(…) 任意参数
     * <p>
     *
     * @param pjp：切入点对象
     * @return: java.lang.Object
     */
    @Around(value = "@annotation(com.cloud.framework.core.annotation.RequestSubmitToken)")
    public Object doAround(ProceedingJoinPoint pjp) {
        try {
            HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes())).getRequest();
            Signature signature = pjp.getSignature();
            MethodSignature methodSignature = (MethodSignature) signature;
            //获取目标方法
            Method targetMethod = methodSignature.getMethod();
            //判断有没有统一事务管理注解
            if (targetMethod.isAnnotationPresent(RequestSubmitToken.class)) {
                RequestSubmitToken requestSubmitToken = targetMethod.getAnnotation(RequestSubmitToken.class);
                // 拿到ip地址、请求路径、token
                String ip = IpUtil.getRequestIp(request);
                // 现在时间
                long now = System.currentTimeMillis();
                // 自定义key值方式
                String key = "REQUEST_FORM_" + ip;
                if (redisUtil.hasKey(key)) {
                    // 上次表单提交时间
                    long lastTime = Long.parseLong(redisUtil.get(key).toString());
                    // 如果现在距离上次提交时间小于设置的默认时间 则 判断为重复提交  否则 正常提交 -> 进入业务处理
                    if ((now - lastTime) > requestSubmitToken.time()) {
                        // 非重复提交操作 - 重新记录操作时间
                        redisUtil.set(key, String.valueOf(now));
                        // 进入处理业务
                        pjp.proceed();
                        return pjp.proceed();
                    } else {
                        return ApiResult.fail("请勿重复提交!");
                    }
                } else {
                    // 这里是第一次操作
                    redisUtil.set(key, String.valueOf(now));
                    return pjp.proceed();
                }
            } else {
                return pjp.proceed();
            }
        } catch (Throwable e) {
            log.error("校验表单重复提交时异常: {}", e.getMessage());
            return ApiResult.fail("校验表单重复提交!");
        }

    }

}
