package com.cloud.handler;

import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.exception.AndroidUserException;
import com.cloud.web.exception.AndroidVerifyException;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author sj
 * Date on 2020/9/3  12:45
 * @Version 1.0
 * Android移动端异常处理
 */
@ControllerAdvice
@RestController
@Slf4j
public class AndroidExceptionHandler extends GlobalExceptionHandler {

    /**
     * Android端参数验证异常
     *
     * @param exception
     * @return
     */
    @ExceptionHandler(value = AndroidVerifyException.class)
    @ResponseStatus(HttpStatus.OK)
    public ApiResult VerifyExceptionHandler(AndroidVerifyException exception) {
        super.printRequestDetail();
        super.printApiCodeException(exception.getApiCode(), exception);
        return ApiResult.fail(exception.getApiCode());
    }

    /**
     * Android端用户异常处理
     *
     * @param exception
     * @return
     */
    @ExceptionHandler(value = AndroidUserException.class)
    @ResponseStatus(HttpStatus.OK)
    public ApiResult UserExceptionHandler(AndroidUserException exception) {
        super.printRequestDetail();
        super.printApiCodeException(exception.getApiCode(), exception);
        return ApiResult.fail(exception.getApiCode());
    }

    /**
     * 未知异常处理
     *
     * @param exception
     * @return
     */
    @ExceptionHandler(value = Exception.class)
    @ResponseStatus(HttpStatus.OK)
    public ApiResult UnKnownExceptionHandler(Exception exception) {
        super.printRequestDetail();
        super.printApiCodeException(ApiCode.UNKNOWN_EXCEPTION, exception);
        return ApiResult.fail(ApiCode.UNKNOWN_EXCEPTION);
    }

    /**
     * Android端业务异常处理
     *
     * @param exception
     * @return
     */
    @ExceptionHandler(value = AndroidBusinessException.class)
    @ResponseStatus(HttpStatus.OK)
    public ApiResult BusinessExceptionHandler(AndroidBusinessException exception) {
        super.printRequestDetail();
        super.printApiCodeException(exception.getApiCode(), exception);
        return ApiResult.fail(exception.getApiCode());
    }

    /**
     * 参数效验异常
     *
     * @param exception
     * @return
     */
    @ExceptionHandler(value = BindException.class)
    @ResponseStatus(HttpStatus.OK)
    public ApiResult BindExceptionHandler(BindException exception) {
        printRequestDetail();
        BindingResult bindingResult = exception.getBindingResult();
        List<String> list = new ArrayList<>();
        List<FieldError> fieldErrors = bindingResult.getFieldErrors();
        for (FieldError fieldError : fieldErrors) {
            list.add(fieldError.getDefaultMessage());
        }
        printApiCodeException(ApiCode.PARAMETER_EXCEPTION, exception);
        return ApiResult.fails(list.get(0));
    }
}
