package com.cloud.scheduled.util;

import com.cloud.framework.common.api.ApiCode;
import com.cloud.scheduled.config.QuartzTriggerTypeEnums;
import com.cloud.scheduled.entity.QuartzJob;
import com.cloud.scheduled.exception.QuartzRunTimeException;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.impl.triggers.CronTriggerImpl;
import org.quartz.impl.triggers.SimpleTriggerImpl;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

import static org.quartz.TriggerBuilder.newTrigger;

/**
 * @author sj
 * @date 2020/10/7  15:30
 */
@Slf4j
@Component
@SuppressWarnings("all")
public class QuartzManage {

    private static final String JOB_NAME = "TASK_";

    private static final String EXAM_JOB_NAME = "EXAM_";

    @Resource(name = "scheduler")
    private Scheduler scheduler;

    /**
     * 添加cron表达式任务
     *
     * @param quartzJob
     * @throws QuartzRunTimeException
     */
    public void addCronJob(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            // 构建job信息
            JobDetail jobDetail = JobBuilder.newJob(ExecutionJob.class).
                    withIdentity(JOB_NAME + quartzJob.getId()).build();
            //通过触发器名和cron 表达式创建 Trigger
            Trigger cronTrigger = newTrigger()
                    .withIdentity(JOB_NAME + quartzJob.getId())
                    .startNow()
                    .withSchedule(CronScheduleBuilder.cronSchedule(quartzJob.getCronExpression()))
                    .build();
            cronTrigger.getJobDataMap().put(QuartzJob.JOB_KEY, quartzJob);
            //重置启动时间
            ((CronTriggerImpl) cronTrigger).setStartTime(new Date());
            //执行定时任务
            scheduler.scheduleJob(jobDetail, cronTrigger);
            // 暂停任务
            if (quartzJob.getIsPause()) {
                pauseJob(quartzJob);
            }
        } catch (Exception e) {
            log.error("创建定时任务失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_CREATE_ERR);
        }
    }

    public void addSimpleJob(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            // 构建job信息
            JobDetail jobDetail = JobBuilder.newJob(ExecutionJob.class).
                    withIdentity(JOB_NAME + quartzJob.getId(), EXAM_JOB_NAME).build();
            //通过触发器名
            Trigger simpleTrigger = newTrigger()
                    .withIdentity(JOB_NAME + quartzJob.getId(), EXAM_JOB_NAME)
                    .build();
            simpleTrigger.getJobDataMap().put(QuartzJob.JOB_KEY, quartzJob);
            SimpleTriggerImpl trigger = ((SimpleTriggerImpl) simpleTrigger);
            //重置启动时间
            trigger.setStartTime(quartzJob.getExecuteStartTime() == null ? new Date() : quartzJob.getExecuteStartTime());
//            //设置定时任务结束时间
//            trigger.setEndTime(quartzJob.getEndTime());
            //执行定时任务
            scheduler.scheduleJob(jobDetail, simpleTrigger);
            // 暂停任务
            if (quartzJob.getIsPause()) {
                pauseJob(quartzJob);
            }
        } catch (Exception e) {
            log.error("创建定时任务失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_CREATE_ERR);
        }
    }

    /**
     * 更新job cron表达式
     *
     * @param quartzJob /
     */
    public void updateJobCron(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(JOB_NAME + quartzJob.getId(), EXAM_JOB_NAME);
            Trigger trigger = scheduler.getTrigger(triggerKey);
            // 如果不存在则创建一个定时任务
            if (trigger == null) {
                if (quartzJob.getTriggerType() == 1) {
                    addSimpleJob(quartzJob);
                } else {
                    addCronJob(quartzJob);
                }
            } else {
                if (quartzJob.getTriggerType() == 1) {
                    trigger = trigger.getTriggerBuilder().withIdentity(triggerKey).build();
                    //重置启动时间
                    ((SimpleTriggerImpl) trigger).setStartTime(quartzJob.getExecuteStartTime() == null ? new Date() : quartzJob.getExecuteStartTime());
                    //设置定时任务结束时间
                    if (quartzJob.getEndTime() != null) {
                        ((SimpleTriggerImpl) trigger).setEndTime(quartzJob.getEndTime());
                    }
                    trigger.getJobDataMap().put(QuartzJob.JOB_KEY, quartzJob);
                    scheduler.rescheduleJob(triggerKey,(SimpleTrigger)trigger);
                    // 暂停任务
                    if (quartzJob.getIsPause()) {
                        pauseJob(quartzJob);
                    }
                } else {
                    CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(quartzJob.getCronExpression());
                    trigger = ((CronTriggerImpl) trigger).getTriggerBuilder().withIdentity(triggerKey).withSchedule(scheduleBuilder).build();
                    //重置启动时间
                    ((CronTriggerImpl) trigger).setStartTime(new Date());
                    trigger.getJobDataMap().put(QuartzJob.JOB_KEY, quartzJob);
                    scheduler.rescheduleJob(triggerKey,trigger);
                    // 暂停任务
                    if (quartzJob.getIsPause()) {
                        pauseJob(quartzJob);
                    }
                }
            }

        } catch (Exception e) {
            log.error("更新定时任务失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_UPDATE_ERR);
        }

    }

    /**
     * 删除一个job
     *
     * @param quartzJob /
     */
    public void deleteJob(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            JobKey jobKey = JobKey.jobKey(JOB_NAME + quartzJob.getId());
            scheduler.pauseJob(jobKey);
            scheduler.deleteJob(jobKey);
        } catch (Exception e) {
            log.error("删除定时任务失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_DELETE_ERR);
        }
    }

    /**
     * 恢复一个job
     *
     * @param quartzJob /
     */
    public void resumeJob(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(JOB_NAME + quartzJob.getId());
            CronTrigger trigger = (CronTrigger) scheduler.getTrigger(triggerKey);
            // 如果不存在则创建一个定时任务
            if (trigger == null) {
                if (quartzJob.getTriggerType() == QuartzTriggerTypeEnums.SIMPLE_TRIGGER.getCode()) {
                    addSimpleJob(quartzJob);
                } else {
                    addCronJob(quartzJob);
                }
            }
            JobKey jobKey = JobKey.jobKey(JOB_NAME + quartzJob.getId());
            scheduler.resumeJob(jobKey);
        } catch (Exception e) {
            log.error("恢复定时任务失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_RECOVER_ERR);
        }
    }

    /**
     * 立即执行job
     *
     * @param quartzJob /
     */
    public void runJobNow(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            TriggerKey triggerKey = TriggerKey.triggerKey(JOB_NAME + quartzJob.getId());
            CronTrigger trigger = (CronTrigger) scheduler.getTrigger(triggerKey);
            // 如果不存在则创建一个定时任务
            if (trigger == null) {
                if (quartzJob.getTriggerType() == QuartzTriggerTypeEnums.SIMPLE_TRIGGER.getCode()) {
                    addSimpleJob(quartzJob);
                } else {
                    addCronJob(quartzJob);
                }
            }
            JobDataMap dataMap = new JobDataMap();
            dataMap.put(QuartzJob.JOB_KEY, quartzJob);
            JobKey jobKey = JobKey.jobKey(JOB_NAME + quartzJob.getId());
            scheduler.triggerJob(jobKey, dataMap);
        } catch (Exception e) {
            log.error("定时任务执行失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_RUN_ERR);
        }
    }

    /**
     * 暂停一个job
     *
     * @param quartzJob /
     */
    public void pauseJob(QuartzJob quartzJob) throws QuartzRunTimeException {
        try {
            JobKey jobKey = JobKey.jobKey(JOB_NAME + quartzJob.getId());
            scheduler.pauseJob(jobKey);
        } catch (Exception e) {
            log.error("定时任务暂停失败", e);
            throw new QuartzRunTimeException(ApiCode.QUARTZ_PAUSE_ERR);
        }
    }
}
