package com.cloud.system.controller;

import com.alibaba.fastjson.JSON;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLogIgnore;
import com.cloud.framework.shiro.util.JwtTokenUtil;
import com.cloud.framework.shiro.vo.LoginSysUserVo;
import com.cloud.framework.util.LoginUtil;
import com.cloud.system.convert.SysUserConvert;
import com.cloud.system.entity.SysDepartment;
import com.cloud.system.entity.SysRole;
import com.cloud.system.entity.SysUser;
import com.cloud.system.enums.StateEnum;
import com.cloud.system.mapper.SysUserMapper;
import com.cloud.system.param.LoginParam;
import com.cloud.system.service.*;
import com.cloud.system.vo.LoginSysUserTokenVo;
import com.cloud.system.vo.RouterVo;
import com.cloud.system.vo.SysPermissionTreeVo;
import com.cloud.system.vo.SysUserQueryVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Set;

/**
 * 登录控制器
 *
 * @author cyty
 * @date 2019-09-28
 * @since 1.3.0.RELEASE
 **/
@Slf4j
@RestController
@Module("system")
@Api(value = "系统登录API", tags = {"系统登录"})
public class LoginController {

    @Autowired
    private LoginService loginService;

    @Autowired
    private SysDepartmentService sysDepartmentService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysRolePermissionService sysRolePermissionService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private SysPermissionService sysPermissionService;

    @Autowired
    private SysUserMapper sysUserMapper;

    @PostMapping("/login")
    @OperationLogIgnore
    @ApiOperation(value = "登录", notes = "系统用户登录", response = LoginSysUserTokenVo.class)
    public ApiResult<LoginSysUserTokenVo> login(@Validated @RequestBody LoginParam loginParam, HttpServletResponse response) throws Exception {
        LoginSysUserTokenVo loginSysUserTokenVo = loginService.login(loginParam);
        // 设置token响应头
        response.setHeader(JwtTokenUtil.getTokenName(), loginSysUserTokenVo.getToken());
        return ApiResult.ok(loginSysUserTokenVo, "登录成功");
    }


    /**
     * 根据token获取系统登录用户信息
     *
     * @return
     */
    @GetMapping("/getSysUserInfo")
    @ApiOperation(value = "根据token获取系统登录用户信息", response = SysUserQueryVo.class)
    public ApiResult<JSON> getSysUser() throws Exception {
//        String token =  JwtTokenUtil.getToken();
//        String tokenSha256 = DigestUtils.sha256Hex(token);
//        LoginSysUserVo loginSysUserVo = (LoginSysUserVo) redisTemplate.opsForValue().get(tokenSha256);
//        return ApiResult.ok(loginSysUserVo);

        String json = "{\n" +
                "    roles: ['admin'],\n" +
                "    introduction: 'I am a super administrator',\n" +
                "    avatar: 'https://wpimg.wallstcn.com/f778738c-e4f8-4870-b634-56703b4acafe.gif',\n" +
                "    name: 'Super Admin'\n" +
                "  }";
        JSON array = JSON.parseObject(json);

        return ApiResult.ok(array);
    }

    @PostMapping("/getInfo")
    @ApiOperation(value = "登陆", notes = "系统用户登陆", response = ApiResult.class)
    public ApiResult getInfo() throws Exception {

        SysUser sysUser = sysUserMapper.selectById(LoginUtil.getUserId());

        // 将系统用户对象转换成登陆用户对象
        LoginSysUserVo loginSysUserVo = SysUserConvert.INSTANCE.sysUserToLoginSysUserVo(sysUser);

        // 获取部门
        SysDepartment sysDepartment = sysDepartmentService.getById(sysUser.getDepartmentId());
        if (sysDepartment == null) {
            throw new AuthenticationException("部门不存在");
        }
        if (!StateEnum.ENABLE.getCode().equals(sysDepartment.getState())) {
            throw new AuthenticationException("部门已禁用");
        }
        loginSysUserVo.setDepartmentId(sysDepartment.getId())
                .setDepartmentName(sysDepartment.getName());

        // 获取当前用户角色
        Long roleId = sysUser.getRoleId();
        SysRole sysRole = sysRoleService.getById(roleId);
        if (sysRole == null) {
            throw new AuthenticationException("角色不存在");
        }
        if (StateEnum.DISABLE.getCode().equals(sysRole.getState())) {
            throw new AuthenticationException("角色已禁用");
        }
        loginSysUserVo.setRoleId(sysRole.getId())
                .setRoleName(sysRole.getName())
                .setRoleCode(sysRole.getCode()).setHead(sysUser.getHead());

        // 获取当前用户权限
        Set<String> permissionCodes = sysRolePermissionService.getPermissionCodesByRoleId(roleId);
        if (CollectionUtils.isEmpty(permissionCodes)) {
            throw new AuthenticationException("权限列表不能为空");
        }
        loginSysUserVo.setPermissionCodes(permissionCodes);

        return ApiResult.ok(loginSysUserVo);
    }


    @PostMapping("/logout")
    @OperationLogIgnore
    public ApiResult<String> logout(HttpServletRequest request) throws Exception {
        loginService.logout(request);
        return ApiResult.ok("退出成功");
    }

    //获取路由
    @PostMapping("/getRouters")
    public ApiResult getRouters(HttpServletRequest request) throws Exception {
        SysUser sysUser = sysUserMapper.selectById(LoginUtil.getUserId());

        List<SysPermissionTreeVo> tree = sysPermissionService.getMenuTreeByUserId(sysUser.getId());

        List<RouterVo> routerVos = sysPermissionService.buildMenus(tree);

        return ApiResult.ok(routerVos);
    }
}
