package com.cloud.system.service.impl;

import cn.hutool.core.io.FileUtil;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSClientBuilder;
import com.cloud.config.properties.CosProperties;
import com.cloud.config.properties.OssProperties;
import com.cloud.framework.util.RegularUtil;
import com.cloud.framework.util.UUIDUtil;
import com.cloud.system.service.UpLoadService;
import com.qcloud.cos.COSClient;
import com.qcloud.cos.ClientConfig;
import com.qcloud.cos.auth.BasicCOSCredentials;
import com.qcloud.cos.auth.COSCredentials;
import com.qcloud.cos.model.ObjectMetadata;
import com.qcloud.cos.model.PutObjectRequest;
import com.qcloud.cos.model.PutObjectResult;
import com.qcloud.cos.region.Region;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * <pre>
 * 上传文件到腾讯云对象存储
 * </pre>
 *
 * @author cyty
 * @since 2019-10-24
 */
@Slf4j
@Service
public class UpLoadServiceImpl implements UpLoadService {

    @Autowired
    private CosProperties cosProperties;
    @Autowired
    private OssProperties ossProperties;

    @Override
    public String upToOss(MultipartFile multipartFile) throws IOException {
        // 1 初始化用户身份信息（secretId, secretKey）。
        String secretId = cosProperties.getSecretId();
        String secretKey = cosProperties.getSecretKey();
        COSCredentials cred = new BasicCOSCredentials(secretId, secretKey);
        // 2 设置 bucket 的区域, COS 地域的简称请参照 https://cloud.tencent.com/document/product/436/6224
        // clientConfig 中包含了设置 region, https(默认 http), 超时, 代理等 set 方法, 使用可参见源码或者常见问题 Java SDK 部分。
        Region region = new Region("ap-chengdu");
        ClientConfig clientConfig = new ClientConfig(region);
        // 3 生成 cos 客户端。
        COSClient cosClient = new COSClient(cred, clientConfig);
        // 指定要上传到的存储桶
        String bucketName = cosProperties.getBucket();
        // 指定要上传到 COS 上对象键
        //文件保存名称

        // 文件后缀
        String fileExtension = FilenameUtils.getExtension(multipartFile.getOriginalFilename());
        // 这里可自定义文件名称，比如按照业务类型/文件格式/日期
        String dateString = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmssS")) + RandomStringUtils.randomNumeric(6);
        String fileName = dateString + "." + fileExtension;

        String key = "/upload/" + fileName;

        ObjectMetadata objectMetadata = new ObjectMetadata();
        objectMetadata.setContentLength(multipartFile.getSize());//文件的大小
        objectMetadata.setContentType(multipartFile.getContentType());//文件的类型
        PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, key, multipartFile.getInputStream(), objectMetadata);
        PutObjectResult putObjectResult = cosClient.putObject(putObjectRequest);

        System.out.println(putObjectResult);
        // https://ai-1257235067.cos.ap-chengdu.myqcloud.com/upload/202007091854473837451.jpg
        String url = "https://" + bucketName + ".cos.ap-chengdu.myqcloud.com" + key;
        return url;
    }

    /**
     * 上传文件到阿里云对象存储
     *
     * @param multipartFile
     * @return
     */
    @Override
    public String uploadToOOS(MultipartFile multipartFile) throws IOException {
        // 获取输入流
        InputStream inputStream = multipartFile.getInputStream();
        // 文件后缀
        String fileExtension = FilenameUtils.getExtension(multipartFile.getOriginalFilename());
        // 这里可自定义文件名称，比如按照业务类型/文件格式/日期
        String dateString = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmssS")) + RandomStringUtils.randomNumeric(6);
        String fileName = dateString + "." + fileExtension;

        String file = "upload/" + fileName;

        //阿里云对象存储配置
        String endpoint = ossProperties.getEndpoint();
        String accessKeyId = ossProperties.getAccessKeyId();
        String accessKeySecret = ossProperties.getAccessKeySecret();
        String bucket = ossProperties.getBucket();

        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        ossClient.putObject(bucket, file, inputStream);
        // 关闭OSSClient。
        ossClient.shutdown();
        //返回文件路径
        String url = "https://" + bucket + "." + RegularUtil.convertUrl(endpoint) + "/" + file;
        return url;
    }

    /**
     * 上传文件到阿里云对象存储
     *
     * @param bytes
     * @return
     */
    @Override
    public String uploadToOOS(byte[] bytes) throws IOException {
        String file = "upload/" + 123;
        //阿里云对象存储配置
        String endpoint = ossProperties.getEndpoint();
        String accessKeyId = ossProperties.getAccessKeyId();
        String accessKeySecret = ossProperties.getAccessKeySecret();
        String bucket = ossProperties.getBucket();
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        ossClient.putObject(bucket, file, new ByteArrayInputStream(bytes));
        ossClient.shutdown();
        //返回文件路径
        String url = "https://" + bucket + "." + RegularUtil.convertUrl(endpoint) + "/" + file;
        return url;
    }


    @Override
    public boolean deleteFile(String filePath) {
        //阿里云对象存储配置
        String endpoint = ossProperties.getEndpoint();
        String accessKeyId = ossProperties.getAccessKeyId();
        String accessKeySecret = ossProperties.getAccessKeySecret();
        String bucket = ossProperties.getBucket();
        // 创建OSSClient实例。
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
//        boolean exist = ossClient.doesObjectExist(bucket, filePath);
//        if (!exist) {
//            log.error("文件不存在,filePath={}", filePath);
//            return false;
//        }
        log.info("删除文件,filePath={}", filePath);
        ossClient.deleteObject(bucket, filePath);
        ossClient.shutdown();
        return true;
    }

}
